/***********************************************************************
 * Linotte                                                             *
 * Version release date : October 5, 2011                              *
 * Author : Mounes Ronan ronan.mounes@amstrad.eu                       *
 *                                                                     *
 *     http://langagelinotte.free.fr                                   *
 *                                                                     *
 * This code is released under the GNU GPL license, version 2 or       *
 * later, for educational and non-commercial purposes only.            *
 * If any part of the code is to be included in a commercial           *
 * software, please contact us first for a clearance at                *
 *   ronan.mounes@amstrad.eu                                           *
 *                                                                     *
 *   This notice must remain intact in all copies of this code.        *
 *   This code is distributed WITHOUT ANY WARRANTY OF ANY KIND.        *
 *   The GNU GPL license can be found at :                             *
 *           http://www.gnu.org/copyleft/gpl.html                      *
 *                                                                     *
 ***********************************************************************/

package org.linotte.greffons;

import static org.linotte.moteur.xml.actions.EtatTilde.STATUS.A_JOUR;
import static org.linotte.moteur.xml.actions.EtatTilde.STATUS.METTRE_A_JOUR;
import static org.linotte.moteur.xml.actions.EtatTilde.STATUS.NOUVEAU;

import java.awt.Frame;
import java.awt.Toolkit;
import java.awt.event.WindowEvent;
import java.io.BufferedOutputStream;
import java.io.File;
import java.math.BigDecimal;
import java.net.URL;
import java.net.URLConnection;
import java.util.List;
import java.util.Scanner;

import javax.swing.JOptionPane;

import org.linotte.frame.latoile.LaToile;
import org.linotte.greffons.api.Greffon;
import org.linotte.greffons.externe.Graphique;
import org.linotte.greffons.ihm.GreffonBar;
import org.linotte.greffons.java.GreffonPrototype;
import org.linotte.greffons.java.GreffonPrototype.Entrée;
import org.linotte.moteur.entites.Acteur;
import org.linotte.moteur.entites.Prototype;
import org.linotte.moteur.entites.PrototypeGraphique;
import org.linotte.moteur.entites.Role;
import org.linotte.moteur.outils.Preference;
import org.linotte.moteur.outils.Ressources;
import org.linotte.moteur.xml.Linotte;
import org.linotte.moteur.xml.actions.EtatTilde.STATUS;
import org.linotte.moteur.xml.alize.kernel.RuntimeContext;
import org.linotte.moteur.xml.api.Librairie;

/**
 * Chargement de greffons depuis la commande tilde greffon
 * @author CPC
 *
 */
public class GestionDesGreffons {

	public static void chargerGreffon(final String commande, final RuntimeContext runtimeContext) throws Exception {
		chargerGreffon(commande, runtimeContext.getLinotte(), true);
	}

	public static boolean chargerGreffon(final String commande, final Linotte linotte, boolean texteLivre) throws Exception {
		Scanner scanner = new Scanner(commande);
		try {
			@SuppressWarnings("unused")
			int version_protocole = scanner.nextInt();
			//System.out.println("GDG : Version protocole " + version_protocole);

			String nom = scanner.next();
			Greffon greffon_temp = GreffonsChargeur.getInstance().getGreffon(nom);
			double versionDemandee = Double.valueOf(scanner.next());
			STATUS etat;
			if (greffon_temp == null) {
				// Greffon n'est pas present :
				etat = NOUVEAU;
			} else {
				// Mise à jour d'un greffon :
				double versionActuelle = Double.valueOf(greffon_temp.getVersion());
				etat = versionDemandee > versionActuelle ? METTRE_A_JOUR : A_JOUR;
				//runtimeContext.getIhm().afficherErreur("Demande de mise à jour du greffon !");
			}

			if (etat != A_JOUR) {
				StringBuffer t = new StringBuffer();
				while (scanner.hasNext()) {
					t.append(scanner.next());
				}
				// On supprime les guillemets :
				URL url = new URL(t.substring(1, t.length() - 1));
				Object[] options = { etat == NOUVEAU ? "Installer" : "Mettre à jour", "Non, merci" };
				int n = JOptionPane.showOptionDialog(null, (texteLivre ? ("Ce livre a besoin "
						+ (etat == NOUVEAU ? "de l'installation d'un nouveau" : "d'une mise à jour d'un") + " greffon.\n") : "")
						+ "Voulez-vous télécharger la dernière version du greffon " + nom.toUpperCase() + " depuis l'URL :\n" + url.toURI().toString(),
						"Gestionnaire des greffons", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE,
						Ressources.getImageIcon("applications-other.png"), options, options[1]);

				if (n == 0) {
					URLConnection connection = url.openConnection();
					int length = connection.getContentLength();
					System.out.println("GDG : Taille " + length);
					System.out.println("GDG : Type " + connection.getContentType());
					String f = extractFileName(connection.getURL().getFile());
					System.out.println("GDG : Nom du fichier " + f);

					File pere = new File(Preference.getIntance().getHome() + File.separator + Preference.REPERTOIRE + File.separator + GreffonsHandler.DIR_USER);
					pere.mkdirs();

					GreffonBar bar = GreffonBar.createAndShowGUI(length);

					File out = new File(pere, f);

					try {

						java.io.BufferedInputStream in = new java.io.BufferedInputStream(connection.getInputStream());
						java.io.FileOutputStream fos = new java.io.FileOutputStream(out);
						java.io.BufferedOutputStream bout = new BufferedOutputStream(fos, 1024);
						// Téléchargement
						byte[] data = new byte[1024];
						int x = 0;
						while ((x = in.read(data, 0, 1024)) >= 0) {
							bout.write(data, 0, x);
							bar.onAvanceDe(x);
							Thread.sleep(10); // smooth it !
						}
						bout.close();
						in.close();

					} finally {
						bar.getJDialog().dispose();
					}

					Librairie lib = linotte.getLibrairie();

					if (etat == NOUVEAU) {
						List<Greffon> greffons = GreffonsHandler.chargerGreffonGZL(out);
						for (Greffon greffon : greffons) {

							greffon = GreffonsChargeur.getInstance().getGreffon(greffon.getId().toLowerCase());

							// On enregistre ce greffon au niveau de l'interprete :
							try {
								if (greffon.getObjet() instanceof Graphique) {
									Acteur position = new Acteur(null, "position", Role.NOMBRE, new BigDecimal(0), null);
									Acteur visible = new Acteur(null, "visible", Role.TEXTE, "non", null);

									PrototypeGraphique espece = new PrototypeGraphique(null, lib, greffon.getId(), Role.ESPECE, greffon.getId(), null,
											PrototypeGraphique.TYPE_GRAPHIQUE.GREFFON, GreffonsChargeur.getInstance().newInstance(greffon.getId()));
									if (!PrototypeGraphique.isEspecesGraphique_(greffon.getId()))
										linotte.especeModeleMap.add(espece);
									lib.addEspece(espece);
									GreffonPrototype.chargeAttributGreffon(espece, greffon);
									espece.addAttribut(visible);
									espece.addAttribut(position);
									// Ajout des méthodes fonctionnelles :
									for (Entrée entrée : greffon.getSlots()) {
										espece.ajouterSlotGreffon(entrée.getKey(), entrée.getValue());
									}
								} else if (greffon.getObjet() instanceof org.linotte.greffons.externe.Greffon) {
									Prototype espece = new Prototype(lib, greffon.getId(), Role.ESPECE, greffon.getId(), null, GreffonsChargeur.getInstance()
											.newInstance(greffon.getId()));
									if (!PrototypeGraphique.isEspecesGraphique_(greffon.getId()))
										linotte.especeModeleMap.add(espece);
									lib.addEspece(espece);
									GreffonPrototype.chargeAttributGreffon(espece, greffon);
									// Ajout des méthodes fonctionnelles :
									for (Entrée entrée : greffon.getSlots()) {
										espece.ajouterSlotGreffon(entrée.getKey(), entrée.getValue());
									}
								}
							} catch (Exception e) {
								e.printStackTrace();
							}

						}

					} else {
						JOptionPane.showMessageDialog(null,
								"Le téléchargement a été effectué. Linotte va fermer.\nMerci de le relancer pour prendre en compte le nouveau greffon.");
						Frame frame = null;

						try {
							LaToile toile = lib.getToilePrincipale();
							frame = toile.getFrameParent();
						} catch (Exception e) {
						}

						if (frame != null) {
							WindowEvent wev = new WindowEvent(frame, WindowEvent.WINDOW_CLOSING);
							Toolkit.getDefaultToolkit().getSystemEventQueue().postEvent(wev);
						} else {
							System.exit(0);
						}
					}
				} else {
					return false;
				}

			}
		} finally {
			scanner.close();
		}
		return true;
	}

	private static String extractFileName(String path) {

		if (path == null) {
			return null;
		}
		String newpath = path.replace('\\', '/');
		int start = newpath.lastIndexOf("/");
		if (start == -1) {
			start = 0;
		} else {
			start = start + 1;
		}
		String pageName = newpath.substring(start, newpath.length());

		return pageName;
	}

}
