package org.linotte.web.serveur;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.alize.http.i.Executeur;
import org.linotte.moteur.outils.Preference;
import org.linotte.moteur.outils.Ressources;
import org.linotte.moteur.xml.Version;
import org.linotte.web.Run;
import org.mortbay.jetty.Request;
import org.mortbay.jetty.Server;
import org.mortbay.jetty.handler.AbstractHandler;

public class JettyHttpServer extends AbstractServeur {

	private String version;

	private static JettyHttpServer _this;

	private File notice;

	private JettyHttpServer(String pversion) {
		version = pversion;
	}

	public static synchronized JettyHttpServer getServeur(String pversion) {
		return (_this == null) ? _this = new JettyHttpServer(pversion) : _this;
	}

	public void start(int port, File root, Executeur executeur) throws Exception {
		if (!Version.isBeta()) System.setProperty("org.mortbay.log.class", "org.linotte.web.serveur.FakeLogger");
		Server server = new Server(port);
		server.setHandler(new Handler(executeur, root));
		server.start();

		if (Version.isBeta()) System.out.println(version + " démarré sur le port " + port);

		// Creation de la notice : 
		InputStream in = Ressources.getFlux("notice.liv");
		notice = File.createTempFile("notice", ".liv");
		notice.deleteOnExit();
		if (Version.isBeta()) System.out.println("creation du fichier temporaire : " + notice);
		FileOutputStream out = new FileOutputStream(notice);
		copy(in, out);
	}

	public void stop() throws IOException {
	}

	private class Handler extends AbstractHandler {

		private Executeur executeur;

		private File root;

		public Handler(Executeur pexecuteur, File proot) {
			executeur = pexecuteur;
			root = proot;
		}

		@SuppressWarnings("unchecked")
		public void handle(String target, HttpServletRequest request, HttpServletResponse res, int dispatch) throws IOException,
				ServletException {

			long t0 = System.currentTimeMillis();
			StringBuilder response = new StringBuilder();
			int code = HttpServletResponse.SC_OK;

			// Livre : 
			String path = request.getRequestURI();

			try {
				if (executeur.accepter(path)) {

					File livre;
					if ("/notice".equals(path)) {
						// Filtre Notice : 
						livre = notice;
					} else if (path.startsWith("/b_tutoriel")) {
						// Filtre turoriel : 
						livre = new File(Ressources.getTutoriel(), path.substring(9));
					} else {
						livre = new File(root, path);
					}

					Map<String, String> acteurs = new HashMap<String, String>();
					Enumeration<String> e = request.getParameterNames();
					while (e.hasMoreElements()) {
						String s = e.nextElement();
						acteurs.put(s, request.getParameter(s));
					}

					// Ajout des parametres du serveur : 
					acteurs.put("wbt_repertoire", root.getCanonicalPath());
					acteurs.put("wbt_port", Integer.toString(Run.PORT));

					// Exécution de la requête 
					response.append(executeur.traiter(livre, acteurs));
				} else {
					throw new SecurityException("Type de fichier interdit");
				}
			} catch (Exception e) {
				response.delete(0, response.length());
				response.append("<html><head><title>Message : " + e.toString() + "</title><head><body>");
				afficherErreur(response, e);
				Throwable t = e;
				while (t.getCause() != null) {
					t = t.getCause();
					response.append("<hr/>");
					afficherErreur(response, t);
				}
				response.append("<p>Avez-vous lu la notice ? <a href=\"/notice\">http://localhost:" + Run.PORT + "/notice</a></p>");

				response.append("</body></html>");
				code = HttpServletResponse.SC_INTERNAL_SERVER_ERROR;
			}
			long t1 = System.currentTimeMillis() - t0;
			if (!Preference.getIntance().getBoolean(Preference.P_WEBONOTTE_FOOTER)) {
				response.append("<hr/><div id=\"webofooter\"><p><small><a href=\"http://langagelinotte.free.fr\">" + version
						+ "</a> - Livre lu en " + t1 + " ms - <a href=\"http://localhost:" + Run.PORT + path
						+ "?wbt_source=true\">Voir le livre</a></small></p></div>");

			}

			res.setContentType("text/html");
			res.setStatus(code);
			res.getWriter().println(response.toString());
			((Request) request).setHandled(true);
		}

		private void afficherErreur(StringBuilder response, Throwable e) {
			StringWriter sw = new StringWriter();
			PrintWriter pw = new PrintWriter(sw);
			e.printStackTrace(pw);
			String s = sw.toString().replaceAll("\n", "</br>");
			response.append(s);
			//			if ( e instanceof LectureException ) {
			//				LectureException le = (LectureException) e;
			//				afficherErreur(response, le.getException());
			//			}
		}
	}

}